/* date : 10/12/1994 */

#include <stdio.h>
#include <math.h>
#include <string.h>

#define anint(x)       (floor((x)+.5))
#define EPSILON        0.001
#define grid_sup(i)    (anint(((i)+GRID/2-EPSILON)/GRID)*GRID)
#define grid_inf(i)    (((int)(anint((i)/GRID)))*GRID)
float max(a,b) float a,b ; {return ((a)>(b))?(a):(b);}
#define min(a,b)       (((a)<(b))?(a):(b))
#define equ(a,b)       ((((a)-(b))<=EPSILON)?1:0)

/*
 * ---------------------------------------------------------------------------
 * Physical layer names are valid for few processes.
 * Some physical and symbolic names are the same, but when they are used
 * in equations, they are prefixed the rule type, so it is impossible to
 * to make a error.
 *
 *   NWEL : N WELL
 *   PWEL : P WELL
 *   NIMP : N+ implanted zone
 *   PIMP : P+ implanted zone
 *   ACTI : active zone
 *   POLY : POLY wire or Transistors
 *   VIA0 : CONTACT (cut to join ALU1 to POLX or DIFX)
 *   VIA1 : VIA (cut to join ALU1 to ALU2)
 *   VIA2 : VIA (cut to join ALU3 to ALU2)
 *   ALU1 : ALU first level
 *   ALU2 : ALU second level
 *   ALU3 : ALU third level
 *
 * Real Rule type
 *
 *   RW_LAYER         : Real Width
 *   RD_LAYER	     : Real Distance between two edges of the same layer
 *   RD_LAYER1_LAYER2 : Real Distance between layer1 and layer2
 *   RE_LAYER1_LAYER2 : Real Extension of Layer1 on Layer2
 *
 * GRID is the physical grid, every physical rule is a multiple of it.
 *
 * REMARK : All numbers have to be written with a dot as a float
 * ---------------------------------------------------------------------------
 */

float GRID         = 0.0;

float LEVELS       = 2;

float CAPA_S_POLY  = 0.0;
float CAPA_S_ALU1  = 0.0;
float CAPA_S_ALU2  = 0.0;
float CAPA_P_POLY  = 0.0;
float CAPA_P_ALU1  = 0.0;
float CAPA_P_ALU2  = 0.0;

float RW_NWEL      = 0.0;
float RW_PWEL      = 0.0;
float RW_ACTI      = 0.0;
float RW_NIMP      = 0.0;
float RW_PIMP      = 0.0;
float RW_POLY      = 0.0;
float RW_VIA0      = 0.0;
float RW_VIA1      = 0.0;
float RW_VIA2      = 0.0;
float RW_ALU1      = 0.0;
float RW_ALU2      = 0.0;
float RW_ALU3      = 0.0;

float RD_NWEL      = 0.0;
float RD_PWEL      = 0.0;
float RD_ACTI      = 0.0;
float RD_PIMP      = 0.0;
float RD_NIMP      = 0.0;
float RD_POLY      = 0.0;
float RD_VIA0      = 0.0;
float RD_VIA1      = 0.0;
float RD_VIA2      = 0.0;
float RD_ALU1      = 0.0;
float RD_ALU2      = 0.0;
float RD_ALU3      = 0.0;

float RD_VIA0_VIA1 = 0.0;
float RD_POLY_ACTI = 0.0;
float RD_POLY_VIA0 = 0.0;
float RD_POLY_VIA1 = 0.0;
float RD_NWEL_ACTI = 0.0;
float RD_NWEL_PWEL = 0.0;

float RE_NWEL_PIMP = 0.0;
float RE_NWEL_NTIE = 0.0;
float RE_PWEL_NIMP = 0.0;
float RE_PWEL_PTIE = 0.0;
float RE_NIMP_ACTI = 0.0;
float RE_PIMP_ACTI = 0.0;
float RE_NTIE_ACTI = 0.0;
float RE_PTIE_ACTI = 0.0;
float RE_ACTI_POLY = 0.0;
float RE_POLY_ACTI = 0.0;
float RE_ACTI_VIA0 = 0.0;
float RE_POLY_VIA0 = 0.0;
float RE_ALU1_VIA0 = 0.0;
float RE_ALU1_VIA1 = 0.0;
float RE_ALU2_VIA1 = 0.0;
float RE_ALU2_VIA2 = 0.0;
float RE_ALU3_VIA2 = 0.0;

char * CIF_NWEL    = "?";
char * CIF_PWEL    = "?";
char * CIF_NIMP    = "?";
char * CIF_PIMP    = "?";
char * CIF_ACTI    = "?";
char * CIF_POLY    = "?";
char * CIF_VIA0    = "?";
char * CIF_VIA1    = "?";
char * CIF_VIA2    = "?";
char * CIF_ALU1    = "?";
char * CIF_ALU2    = "?";
char * CIF_ALU3    = "?";
char * CIF_TALU1   = "?";
char * CIF_TALU2   = "?";
char * CIF_TALU3   = "?";
char * CIF_CPAS    = "?";

int GDS_NWEL       = -1;
int GDS_PWEL       = -1;
int GDS_NIMP       = -1;
int GDS_PIMP       = -1;
int GDS_ACTI       = -1;
int GDS_POLY       = -1;
int GDS_VIA0       = -1;
int GDS_VIA1       = -1;
int GDS_VIA2       = -1;
int GDS_ALU1       = -1;
int GDS_ALU2       = -1;
int GDS_ALU3       = -1;
int GDS_TALU1      = -1;
int GDS_TALU2      = -1;
int GDS_TALU3      = -1;
int GDS_CPAS       = -1;

void
  parse ()
{
  char *line;
  char *token;
  char *diese;
  float value;
  int  ivalue;
  char *svalue;

  line = (char *)malloc(255);
  token = (char *)malloc(255);
  svalue = (char *)malloc(255);

  while( gets(line) != NULL) {

    /* remove comments */
    if (diese = strchr(line, '#')) *diese = '\0';

    /* ignore blank lines */
    if (sscanf (line, "%s", token) == -1) continue;     
    
    /* read line */
    value=0.0;
    ivalue=-1;
    strcpy(svalue, "?");
    if (sscanf (line, "%s%f", token, &value)!=2) {
      if (sscanf (line, "%s%s%d", token, svalue, &ivalue)!=3) {
        fprintf( stderr, "*** error : <<%s>> incorrect\n", line);
          continue;
      }
    }

    if (!strcmp(token, "GRID"))         GRID = value;

    if (equ(GRID,0.0)) {
      fprintf( stderr, "*** error : GRID undefined or null\n");
      continue;
    }

    if (!strcmp(token, "LEVELS"))       LEVELS = value;

    if (!strcmp(token, "CAPA_S_POLY"))    CAPA_S_POLY = value;
    if (!strcmp(token, "CAPA_S_ALU1"))    CAPA_S_ALU1 = value;
    if (!strcmp(token, "CAPA_S_ALU2"))    CAPA_S_ALU2 = value;
    if (!strcmp(token, "CAPA_P_POLY"))    CAPA_P_POLY = value;
    if (!strcmp(token, "CAPA_P_ALU1"))    CAPA_P_ALU1 = value;
    if (!strcmp(token, "CAPA_P_ALU2"))    CAPA_P_ALU2 = value;

    if (!strcmp(token, "RW_NWEL"))      RW_NWEL = grid_sup(value) ;
    if (!strcmp(token, "RW_PWEL"))      RW_PWEL = grid_sup(value) ;
    if (!strcmp(token, "RW_ACTI"))      RW_ACTI = grid_sup(value) ;
    if (!strcmp(token, "RW_NIMP"))      RW_NIMP = grid_sup(value) ;
    if (!strcmp(token, "RW_PIMP"))      RW_PIMP = grid_sup(value) ;
    if (!strcmp(token, "RW_POLY"))      RW_POLY = grid_sup(value) ;
    if (!strcmp(token, "RW_VIA0"))      RW_VIA0 = grid_sup(value) ;
    if (!strcmp(token, "RW_VIA1"))      RW_VIA1 = grid_sup(value) ;
    if (!strcmp(token, "RW_VIA2"))      RW_VIA2 = grid_sup(value) ;
    if (!strcmp(token, "RW_ALU1"))      RW_ALU1 = grid_sup(value) ;
    if (!strcmp(token, "RW_ALU2"))      RW_ALU2 = grid_sup(value) ;
    if (!strcmp(token, "RW_ALU3"))      RW_ALU3 = grid_sup(value) ;

    if (!strcmp(token, "RD_NWEL"))      RD_NWEL = grid_sup(value) ;
    if (!strcmp(token, "RD_PWEL"))      RD_PWEL = grid_sup(value) ;
    if (!strcmp(token, "RD_ACTI"))      RD_ACTI = grid_sup(value + GRID*2) ;
    if (!strcmp(token, "RD_PIMP"))      RD_PIMP = grid_sup(value) ;
    if (!strcmp(token, "RD_NIMP"))      RD_NIMP = grid_sup(value) ;
    if (!strcmp(token, "RD_POLY"))      RD_POLY = grid_sup(value) ;

    if (!strcmp(token, "RW_NWEL"))      RW_NWEL = grid_sup(value) ;
    if (!strcmp(token, "RW_PWEL"))      RW_PWEL = grid_sup(value) ;
    if (!strcmp(token, "RW_ACTI"))      RW_ACTI = grid_sup(value) ;
    if (!strcmp(token, "RW_NIMP"))      RW_NIMP = grid_sup(value) ;
    if (!strcmp(token, "RW_PIMP"))      RW_PIMP = grid_sup(value) ;
    if (!strcmp(token, "RW_POLY"))      RW_POLY = grid_sup(value) ;
    if (!strcmp(token, "RW_VIA0"))      RW_VIA0 = grid_sup(value) ;
    if (!strcmp(token, "RW_VIA1"))      RW_VIA1 = grid_sup(value) ;
    if (!strcmp(token, "RW_VIA2"))      RW_VIA2 = grid_sup(value) ;
    if (!strcmp(token, "RW_ALU1"))      RW_ALU1 = grid_sup(value) ;
    if (!strcmp(token, "RW_ALU2"))      RW_ALU2 = grid_sup(value) ;
    if (!strcmp(token, "RW_ALU3"))      RW_ALU3 = grid_sup(value) ;

    if (!strcmp(token, "RD_NWEL"))      RD_NWEL = grid_sup(value) ;
    if (!strcmp(token, "RD_PWEL"))      RD_PWEL = grid_sup(value) ;
    if (!strcmp(token, "RD_ACTI"))      RD_ACTI = grid_sup(value + GRID*2) ;
    if (!strcmp(token, "RD_PIMP"))      RD_PIMP = grid_sup(value) ;
    if (!strcmp(token, "RD_NIMP"))      RD_NIMP = grid_sup(value) ;
    if (!strcmp(token, "RD_POLY"))      RD_POLY = grid_sup(value) ;
    if (!strcmp(token, "RD_VIA0"))      RD_VIA0 = grid_sup(value) ;
    if (!strcmp(token, "RD_VIA1"))      RD_VIA1 = grid_sup(value) ;
    if (!strcmp(token, "RD_VIA2"))      RD_VIA2 = grid_sup(value) ;
    if (!strcmp(token, "RD_ALU1"))      RD_ALU1 = grid_sup(value) ;
    if (!strcmp(token, "RD_ALU2"))      RD_ALU2 = grid_sup(value) ;
    if (!strcmp(token, "RD_ALU3"))      RD_ALU3 = grid_sup(value) ;

    if (!strcmp(token, "RD_VIA0_VIA1")) RD_VIA0_VIA1 = grid_sup(value) ;
    if (!strcmp(token, "RD_POLY_ACTI")) RD_POLY_ACTI = grid_sup(value) ;
    if (!strcmp(token, "RD_POLY_VIA0")) RD_POLY_VIA0 = grid_sup(value) ;
    if (!strcmp(token, "RD_POLY_VIA1")) RD_POLY_VIA1 = grid_sup(value) ;
    if (!strcmp(token, "RD_NWEL_ACTI")) RD_NWEL_ACTI = grid_sup(value) ;
    if (!strcmp(token, "RD_NWEL_PWEL")) RD_NWEL_PWEL = grid_sup(value) ;

    if (!strcmp(token, "RE_NWEL_PIMP")) RE_NWEL_PIMP = grid_sup(value) ;
    if (!strcmp(token, "RE_NWEL_NTIE")) RE_NWEL_NTIE = grid_sup(value) ;
    if (!strcmp(token, "RE_PWEL_NIMP")) RE_PWEL_NIMP = grid_sup(value) ;
    if (!strcmp(token, "RE_PWEL_PTIE")) RE_PWEL_PTIE = grid_sup(value) ;
    if (!strcmp(token, "RE_NIMP_ACTI")) RE_NIMP_ACTI = grid_sup(value) ;
    if (!strcmp(token, "RE_PIMP_ACTI")) RE_PIMP_ACTI = grid_sup(value) ;
    if (!strcmp(token, "RE_NTIE_ACTI")) RE_NTIE_ACTI = grid_sup(value) ;
    if (!strcmp(token, "RE_PTIE_ACTI")) RE_PTIE_ACTI = grid_sup(value) ;
    if (!strcmp(token, "RE_ACTI_POLY")) RE_ACTI_POLY = grid_sup(value) ;
    if (!strcmp(token, "RE_POLY_ACTI")) RE_POLY_ACTI = grid_sup(value) ;
    if (!strcmp(token, "RE_ACTI_VIA0")) RE_ACTI_VIA0 = grid_sup(value) ;
    if (!strcmp(token, "RE_POLY_VIA0")) RE_POLY_VIA0 = grid_sup(value) ;
    if (!strcmp(token, "RE_ALU1_VIA0")) RE_ALU1_VIA0 = grid_sup(value) ;
    if (!strcmp(token, "RE_ALU1_VIA1")) RE_ALU1_VIA1 = grid_sup(value) ;
    if (!strcmp(token, "RE_ALU2_VIA1")) RE_ALU2_VIA1 = grid_sup(value) ;
    if (!strcmp(token, "RE_ALU2_VIA2")) RE_ALU2_VIA2 = grid_sup(value) ;
    if (!strcmp(token, "RE_ALU3_VIA2")) RE_ALU3_VIA2 = grid_sup(value) ;

    if (!strcmp(token,"NAME_NWEL")) {GDS_NWEL=ivalue; CIF_NWEL=strdup(svalue);}
    if (!strcmp(token,"NAME_PWEL")) {GDS_PWEL=ivalue; CIF_PWEL=strdup(svalue);}
    if (!strcmp(token,"NAME_NIMP")) {GDS_NIMP=ivalue; CIF_NIMP=strdup(svalue);}
    if (!strcmp(token,"NAME_PIMP")) {GDS_PIMP=ivalue; CIF_PIMP=strdup(svalue);}
    if (!strcmp(token,"NAME_ACTI")) {GDS_ACTI=ivalue; CIF_ACTI=strdup(svalue);}
    if (!strcmp(token,"NAME_POLY")) {GDS_POLY=ivalue; CIF_POLY=strdup(svalue);}
    if (!strcmp(token,"NAME_VIA0")) {GDS_VIA0=ivalue; CIF_VIA0=strdup(svalue);}
    if (!strcmp(token,"NAME_VIA1")) {GDS_VIA1=ivalue; CIF_VIA1=strdup(svalue);}
    if (!strcmp(token,"NAME_VIA2")) {GDS_VIA2=ivalue; CIF_VIA2=strdup(svalue);}
    if (!strcmp(token,"NAME_ALU1")) {GDS_ALU1=ivalue; CIF_ALU1=strdup(svalue);}
    if (!strcmp(token,"NAME_ALU2")) {GDS_ALU2=ivalue; CIF_ALU2=strdup(svalue);}
    if (!strcmp(token,"NAME_ALU3")) {GDS_ALU3=ivalue; CIF_ALU3=strdup(svalue);}
    if (!strcmp(token,"NAME_TALU1")){GDS_TALU1=ivalue;CIF_TALU1=strdup(svalue);}
    if (!strcmp(token,"NAME_TALU2")){GDS_TALU2=ivalue;CIF_TALU2=strdup(svalue);}
    if (!strcmp(token,"NAME_TALU3")){GDS_TALU3=ivalue;CIF_TALU3=strdup(svalue);}
    if (!strcmp(token,"NAME_CPAS")) {GDS_CPAS=ivalue; CIF_CPAS=strdup(svalue);}

  }
}

/*
 * ---------------------------------------------------------------------------
 * Symbolic layer names, which appear below, are not exactly the MBK names,
 * because this file must contain as less values as possible.
 * Theses names are chosen to avoid to have to give twice the same value.
 *
 *   NWEL : NWELL
 *   DIFR : NDIF, PDIF, NTIE or PTIE in wires
 *   DIFT : NDIF or PDIF in transistors
 *   DIFX : DIFR or DIFT
 *   POLR : POLY wire
 *   POLT : POLY in Transistors
 *   POLX : POLR or POLT
 *   VIA0 : CONTACT (cut to join ALU1 to POLX or DIFX)
 *   VIA1 : VIA (cut to join ALU1 to ALU2)
 *   VIAX : VIA0 or VIA1
 *   ALU1 : ALU first level
 *   ALU2 : ALU second level
 *
 * Symbolic rules type, minimum values (Theses Datas should not be change)
 *
 *   SW_LAYER         : Symbolic Width
 *   SE_LAYER         : Symbolic Extension
 *   SD_LAYER	      : Symbolic Distance between two edges of the same layer
 *   SD_LAYER1_LAYER2 : Symbolic Distance between layer1 and layer2
 *   SE_LAYER1_LAYER2 : Symbolic Extension of Layer1 on Layer2
 * --------------------------------------------------------------------------
 */

float SW_NWEL      = 4.0;
float SW_PWEL      = 4.0;
float SW_DIFX      = 2.0;
float SW_DIFT      = 5.0;
float SW_POLX      = 1.0;
float SW_VIAX      = 1.0;
float SW_VIA2      = 1.0;
float SW_ALU1      = 1.0;
float SW_ALU2      = 2.0;
float SW_ALU3      = 3.0;
float SW_CONT_DIFX = 3.0;

float SE_NWEL      = 0.0;
float SE_DIFT      =-1.5;
float SE_POLR      = 0.5;
float SE_POLT      = 0.0;

float SD_NWEL      =12.0;
float SD_PWEL      =12.0;
float SD_DIFX      = 3.0;
float SD_POLX      = 2.0;
float SD_VIAX      = 3.0;
float SD_ALU1      = 2.5;
float SD_ALU2      = 2.0;
float SD_ALU3      = 2.0;

float SD_NWEL_DIFX = 7.5;
float SD_POLX_DIFX = 1.0;
float SD_POLX_VIA0 = 2.0;
float SD_POLX_VIA1 = 2.0;
float SD_VIA0_VIA1 = 2.0;

float SE_NWEL_DIFX = 0.5;
float SE_DIFX_POLX = 2.0;
float SE_DIFX_VIA0 = 1.0;
float SE_POLX_VIA0 = 1.0;
float SE_ALU1_VIAX = 0.5;
float SE_ALU2_VIA1 = 1.0;
float SE_ALU2_VIA2 = 1.0;
float SE_ALU3_VIA2 = 1.0;

/* pour se ramener a alu1 4 4 4 */
/* 
float SD_ALU1      = 2.0;
float SW_ALU1      = 2.0;
*/

/*
 * ----------------------------------------------------------------------------
 * this function returns the lambda value from a set of constraints
 * ----------------------------------------------------------------------------
 */

float
  lambda_def ()
{
  int   L = 0, i, bubble;
  float LAMBDA_E[255],      E ;
  char  LAMBDA_C[255][255], C[255] ;

/*
 * Lambda definition a set of equations which define constraints
 */

  strcpy( LAMBDA_C[L], "Pitch run NWEL - run NWEL");
  LAMBDA_E[L++] = (RD_NWEL+RW_NWEL)
                / ceil(SD_NWEL+SW_NWEL) ;

  strcpy( LAMBDA_C[L], "Pitch run PWEL - run  PWEL");
  LAMBDA_E[L++] = (RD_PWEL+RW_PWEL)
                / ceil(SD_PWEL+SW_PWEL) ;



  strcpy( LAMBDA_C[L], "Pitch run DIF - run DIF");
  LAMBDA_E[L++] = max(
                   (RD_ACTI+RW_ACTI)
                   / ceil(SD_DIFX+SW_DIFX),
                   (RD_ACTI+2*RE_ACTI_VIA0+RW_VIA0)
                   / ceil(SD_DIFX+SW_CONT_DIFX)
                 );

  strcpy( LAMBDA_C[L], "Pitch run DIF - contact DIF on VIA0");
  LAMBDA_E[L++] = (RW_VIA0/2+RE_ACTI_VIA0+RD_ACTI+RW_ACTI/2)
                / ceil(SW_VIAX/2+SE_DIFX_VIA0+SD_DIFX+SW_DIFX/2) ;

  strcpy( LAMBDA_C[L], "Pitch contact DIF on VIA0 - contact DIF on VIA0");
  LAMBDA_E[L++] = (RW_VIA0+2*RE_ACTI_VIA0+RD_ACTI)
                / ceil(SW_VIAX+2*SE_DIFX_VIA0+SD_DIFX) ;

  strcpy( LAMBDA_C[L], "Pitch run TRANSISTOR - run TRANSISTOR");
  LAMBDA_E[L++] = (RE_ACTI_POLY+RW_POLY)
                / ceil(SE_DIFX_POLX+SW_POLX) ;

  strcpy( LAMBDA_C[L], "Pitch run TRANSISTOR - cont - run TRANSISTOR");
  LAMBDA_E[L++] = (2*RD_POLY_VIA0+RW_VIA0+RW_POLY)
                / ceil(SD_DIFX+2*SD_POLX_VIA0+SW_VIAX+SW_POLX) ;

  strcpy( LAMBDA_C[L], "Pitch contact PDIF on VIA0 - contact NDIF on VIA0");
  LAMBDA_E[L++] = (RW_VIA0/2+RE_ACTI_VIA0+RE_PIMP_ACTI
		  +RE_NWEL_PIMP+RD_NWEL_ACTI+RE_ACTI_VIA0+RW_VIA0/2)
                / ceil(SW_VIAX+2*SE_DIFX_VIA0+SE_NWEL_DIFX+SD_NWEL_DIFX) ;



  strcpy( LAMBDA_C[L], "Pitch run POLY - run POLY");
  LAMBDA_E[L++] = (RD_POLY+RW_POLY)
                / ceil(SD_POLX+SW_POLX) ;

  strcpy( LAMBDA_C[L], "Pitch run POLY - contact POLY on VIA0");
  LAMBDA_E[L++] = (RW_VIA0/2+RE_POLY_VIA0+RD_POLY+RW_POLY/2)
                / ceil(SW_VIAX/2+SE_POLX_VIA0+SD_POLX+SW_POLX/2) ;

  strcpy( LAMBDA_C[L], "Pitch contact POLY on VIA0 - contact POLY on VIA0");
  LAMBDA_E[L++] = (RW_VIA0+2*RE_POLY_VIA0+RD_POLY)
                / ceil(SW_VIAX+2*SE_POLX_VIA0+SD_POLX) ;



  strcpy( LAMBDA_C[L], "Pitch run DIF - run POLY");
  LAMBDA_E[L++] = (RW_ACTI/2+RD_POLY_ACTI+RW_POLY/2)
                / ceil(SW_DIFX/2+SD_POLX_DIFX+SW_POLX/2) ;

  strcpy( LAMBDA_C[L], "Pitch contact DIF on VIA0 - run POLY");
  LAMBDA_E[L++] = (RW_VIA0/2+RE_ACTI_VIA0+RD_POLY_ACTI+RW_POLY/2)
                / ceil(SW_VIAX/2+SE_DIFX_VIA0+SD_POLX_DIFX+SW_POLX/2) ;

  strcpy( LAMBDA_C[L], "Pitch contact DIF on VIA0 - contact POLY on VIA0");
  LAMBDA_E[L++] = (RW_VIA0+RE_ACTI_VIA0+RD_POLY_ACTI+RE_POLY_VIA0)
                / ceil(SW_VIAX+SE_DIFX_VIA0+SD_POLX_DIFX+SE_POLX_VIA0) ;



  strcpy( LAMBDA_C[L], "Pitch run ALU1 - run ALU1");
  LAMBDA_E[L++] = (RD_ALU1+RW_ALU1)
                / ceil(SD_ALU1+SW_ALU1) ;

  strcpy( LAMBDA_C[L], "Pitch run ALU1 - contact ALU1 on VIA0");
  LAMBDA_E[L++] = (RW_VIA0/2+RE_ALU1_VIA0+RD_ALU1+RW_ALU1/2)
                / ceil(SW_VIAX/2+SE_ALU1_VIAX+SD_ALU1+SW_ALU1/2) ;

  strcpy( LAMBDA_C[L], "Pitch contact ALU1 on VIA0 - contact ALU1 on VIA0");
  LAMBDA_E[L++] = (RW_VIA0+2*RE_ALU1_VIA0+RD_ALU1)
                / ceil(SW_VIAX+2*SE_ALU1_VIAX+SD_ALU1) ;

  strcpy( LAMBDA_C[L], "Pitch run ALU1 - contact ALU1 on VIA1");
  LAMBDA_E[L++] = (RW_VIA1/2+RE_ALU1_VIA1+RD_ALU1+RW_ALU1/2)
                / ceil(SW_VIAX/2+SE_ALU1_VIAX+SD_ALU1+SW_ALU1/2) ;

  strcpy( LAMBDA_C[L], "Pitch contact ALU1 on VIA1 - contact ALU1 on VIA1");
  LAMBDA_E[L++] = (RW_VIA1+2*RE_ALU1_VIA1+RD_ALU1 - GRID)
                / ceil(SW_VIAX+2*SE_ALU1_VIAX+SD_ALU1) ;



  strcpy( LAMBDA_C[L], "Pitch run ALU2 - run ALU2");
  LAMBDA_E[L++] = (RD_ALU2+RW_ALU2)
                / ceil(SD_ALU2+SW_ALU2) ;

  strcpy( LAMBDA_C[L], "Pitch run ALU2 - contact ALU2 on VIA1");
  LAMBDA_E[L++] = (RW_VIA1/2+RE_ALU2_VIA1+RD_ALU2+RW_ALU2/2)
                / ceil(SW_VIAX/2+SE_ALU2_VIA1+SD_ALU2+SW_ALU2/2) ;

  strcpy( LAMBDA_C[L], "Pitch contact ALU2 on VIA1 - contact ALU2 on VIA1");
  LAMBDA_E[L++] = (RW_VIA1+2*RE_ALU2_VIA1+RD_ALU2)
                / ceil(SW_VIAX+2*SE_ALU2_VIA1+SD_ALU2) ;



  strcpy( LAMBDA_C[L], "Pitch VIA0 - VIA0");
  LAMBDA_E[L++] = (RW_VIA0+RD_VIA0)
                / ceil(SW_VIAX+SD_VIAX) ;

  strcpy( LAMBDA_C[L], "Pitch VIA1 - VIA1");
  LAMBDA_E[L++] = (RW_VIA1+RD_VIA1)
                / ceil(SW_VIAX+SD_VIAX) ;


if (LEVELS==2) {

  strcpy( LAMBDA_C[L], "Pitch VIA0 - VIA1");
  LAMBDA_E[L++] = (RW_VIA0/2+RD_VIA0_VIA1+RW_VIA1/2)
                / ceil(SW_VIAX+SD_VIA0_VIA1) ;

  strcpy( LAMBDA_C[L], "Pitch cont POLY on VIA0 - VIA1");
  LAMBDA_E[L++] = (RW_VIA0/2+RE_POLY_VIA0+RD_POLY_VIA1+RW_VIA1/2)
                / ceil(SW_VIAX+SE_POLX_VIA0+SD_POLX_VIA1) ;

  strcpy( LAMBDA_C[L], "Pitch run POLY - VIA1");
  LAMBDA_E[L++] = (RW_POLY/2+RD_POLY_VIA1+RW_VIA1/2)
                / ceil(SW_POLX/2+SD_POLX_VIA1+SW_VIAX/2) ;

  strcpy( LAMBDA_C[L], "Pitch contact POLY on VIA0 - contact ALU2 on VIA1");
  LAMBDA_E[L++] = (RW_VIA0/2+RE_POLY_VIA0+RD_POLY_VIA1)
                / ceil(SW_VIAX/2+SE_POLX_VIA0+SD_POLX_VIA1) ;
}

if (LEVELS==3) {

  strcpy( LAMBDA_C[L], "Pitch contact ALU2 on VIA2 - contact ALU2 on VIA2");
  LAMBDA_E[L++] = (RW_VIA2+RE_ALU2_VIA2+RD_ALU2)
                / ceil(SW_VIA2+SE_ALU2_VIA2+SD_ALU2) ;



  strcpy( LAMBDA_C[L], "Pitch run ALU2 - contact ALU2 on VIA2");
  LAMBDA_E[L++] = (RW_VIA2/2+RE_ALU2_VIA2+RD_ALU2+RW_ALU2/2)
                / ceil(SW_VIAX/2+SE_ALU2_VIA2+SD_ALU2+SW_ALU2/2) ;

  strcpy( LAMBDA_C[L], "Pitch contact ALU2 on VIA2 - contact ALU2 on VIA2");
  LAMBDA_E[L++] = (RW_VIA2+2*RE_ALU2_VIA2+RD_ALU2)
                / ceil(SW_VIAX+2*SE_ALU2_VIA2+SD_ALU2) ;



  strcpy( LAMBDA_C[L], "Pitch run ALU3 - run ALU3");
  LAMBDA_E[L++] = (RD_ALU3+RW_ALU3)
                / ceil(SD_ALU3+SW_ALU3) ;

  strcpy( LAMBDA_C[L], "Pitch run ALU3 - contact ALU3 on VIA2");
  LAMBDA_E[L++] = (RW_VIA2/2+RE_ALU3_VIA2+RD_ALU3+RW_ALU3/2)
                / ceil(SW_VIA2/2+SE_ALU3_VIA2+SD_ALU3+SW_ALU3/2) ;

  strcpy( LAMBDA_C[L], "Pitch contact ALU3 on VIA2 - contact ALU3 on VIA2");
  LAMBDA_E[L++] = (RW_VIA2+2*RE_ALU3_VIA2+RD_ALU3)
                / ceil(SW_VIA2+2*SE_ALU3_VIA2+SD_ALU3) ;



  strcpy( LAMBDA_C[L], "Pitch VIA2 - VIA2");
  LAMBDA_E[L++] = (RW_VIA2+RD_VIA2)
                / ceil(SW_VIAX+SD_VIAX) ;
}

/*
 * Lambda computation, sort lambda table to find the biggest value
 */

  for ( bubble = 1 ; bubble ; ) {
    bubble = 0 ;
    for ( i = 0 ; i < L-1 ; i++ ) {
      if (LAMBDA_E[i] < LAMBDA_E[i+1] ) {
	bubble = 1;
	E = LAMBDA_E[i]; LAMBDA_E[i] = LAMBDA_E[i+1] ; LAMBDA_E[i+1] = E;
	strcpy( C, LAMBDA_C[i]);
	strcpy( LAMBDA_C[i], LAMBDA_C[i+1]) ;
	strcpy( LAMBDA_C[i+1], C);
      }
    }
  }
  for ( i = 0 ; i < L ; i++ ) {
    fprintf(stderr, "# Lambda = %5.3f (%f) : %s\n",
           grid_sup(LAMBDA_E[i]),LAMBDA_E[i],LAMBDA_C[i]);
  }

  return grid_sup(LAMBDA_E[0]);
}

/* ----------------------------------------------------------------------- */

/* ODD_LAMBDA <- GRID if LAMBDA is ODD else zero */
#define ODD_LAMBDA ((((int)(anint(LAMBDA/GRID)))&1)*GRID)

#define DW_POLY (RW_POLY-SW_POLX*LAMBDA)
#define DL_POLY grid_inf(RW_POLY/2)

#define DW_ALU1 (RW_ALU1-SW_ALU1*LAMBDA)
#define DL_ALU1 grid_inf(RW_ALU1/2)

#define DW_ALU2 (RW_ALU2-SW_ALU2*LAMBDA)
#define DL_ALU2 grid_inf(RW_ALU2/2)

#define DW_ACTI max(                                                         \
                 RW_ACTI                - SW_DIFX*LAMBDA,                    \
                 RW_VIA0+2*RE_ACTI_VIA0 - SW_CONT_DIFX*LAMBDA                \
                )
#define DL_ACTI_TRANS -1*max(\
			   min (\
			     grid_sup(LAMBDA*SE_DIFT*-1),\
			     grid_sup(RW_VIA0/2+RE_ACTI_VIA0)),\
                           grid_sup(RW_POLY/2+RD_POLY_ACTI))
#define DL_ACTI (2*LAMBDA+DL_ACTI_TRANS)

#define DW_ALU3 (RW_ALU3-SW_ALU3*LAMBDA)
#define DL_ALU3 grid_inf(RW_ALU3/2)

#define DL_NWEL                                                              \
	grid_sup( max( max( max( max ( max ( max (                           \
	  /* transistor _|_ */                                               \
	  RE_NWEL_PIMP+RE_PIMP_ACTI-0.5*LAMBDA,                              \
	  /* contact diffusion */                                            \
	  RW_VIA0/2+RE_ACTI_VIA0+RE_NWEL_PIMP+RE_PIMP_ACTI-2*LAMBDA),        \
	  /* diffusion run _|_ */                                            \
          RW_ACTI/2+RE_NWEL_PIMP+RE_PIMP_ACTI-1.5*LAMBDA),                   \
	  /* diffusion run // or transistor // */                            \
          RE_NWEL_PIMP+RE_PIMP_ACTI-DL_ACTI-LAMBDA),                         \
	  /* contact polarisation */                                         \
	  RW_VIA0/2+RE_ACTI_VIA0+RE_NWEL_NTIE+RE_NTIE_ACTI-2*LAMBDA),        \
	  /* polarisation run _|_ */                                         \
          RW_ACTI/2+RE_NWEL_NTIE+RE_NTIE_ACTI-1.5*LAMBDA),                   \
	  /* polarisation run // */                                          \
          RE_NWEL_NTIE+RE_NTIE_ACTI-DL_ACTI-LAMBDA)                          \
        )

#define DL_PWEL                                                              \
	grid_sup( max( max( max( max ( max ( max (                           \
	  /* transistor _|_ */                                               \
	  RE_PWEL_NIMP+RE_NIMP_ACTI-0.5*LAMBDA,                              \
	  /* contact diffusion */                                            \
	  RW_VIA0/2+RE_ACTI_VIA0+RE_PWEL_NIMP+RE_NIMP_ACTI-2*LAMBDA),        \
	  /* diffusion run _|_ */                                            \
          RW_ACTI/2+RE_PWEL_NIMP+RE_NIMP_ACTI-1.5*LAMBDA),                   \
	  /* diffusion run // or transistor // */                            \
          RE_PWEL_NIMP+RE_NIMP_ACTI-DL_ACTI-LAMBDA),                         \
	  /* contact polarisation */                                         \
	  RW_VIA0/2+RE_ACTI_VIA0+RE_PWEL_PTIE+RE_PTIE_ACTI-2*LAMBDA),        \
	  /* polarisation run _|_ */                                         \
          RW_ACTI/2+RE_PWEL_PTIE+RE_PTIE_ACTI-1.5*LAMBDA),                   \
	  /* polarisation run // */                                          \
          RE_PWEL_PTIE+RE_PTIE_ACTI-DL_ACTI-LAMBDA)                          \
        )

#define DW_NWEL                                                              \
	grid_sup( max( max( max( max ( max ( max (                           \
	  /* transistor _|_ or diffusion run _|_ */                          \
	  2*DL_ACTI+2*RE_PIMP_ACTI+2*RE_NWEL_PIMP-2*LAMBDA,                  \
	  /* contact diffusion */                                            \
	  RW_VIA0+2*RE_ACTI_VIA0+2*RE_PIMP_ACTI+2*RE_NWEL_PIMP-4*LAMBDA),    \
	  /* diffusion run // */                                             \
          RW_ACTI+2*RE_PIMP_ACTI+2*RE_NWEL_PIMP-3*LAMBDA),                   \
	  /* transistor // */                                                \
          2*RE_PIMP_ACTI+2*RE_NWEL_PIMP-LAMBDA),                             \
	  /* contact polarisation */                                         \
	  RW_VIA0+2*RE_ACTI_VIA0+2*RE_NTIE_ACTI+2*RE_NWEL_NTIE-4*LAMBDA),    \
	  /* polarisation run _|_ */                                         \
          2*DL_ACTI+2*RE_NTIE_ACTI+2*RE_NWEL_NTIE-2*LAMBDA),                 \
	  /* polarisation run // */                                          \
          RW_ACTI+2*RE_NTIE_ACTI+2*RE_NWEL_NTIE-3*LAMBDA)                    \
        )

#define DW_PWEL                                                              \
	grid_sup( max( max( max( max ( max ( max (                           \
	  /* transistor _|_ or diffusion run _|_ */                          \
	  2*DL_ACTI+2*RE_NIMP_ACTI+2*RE_PWEL_NIMP-2*LAMBDA,                  \
	  /* contact diffusion */                                            \
	  RW_VIA0+2*RE_ACTI_VIA0+2*RE_NIMP_ACTI+2*RE_PWEL_NIMP-4*LAMBDA),    \
	  /* diffusion run // */                                             \
          RW_ACTI+2*RE_NIMP_ACTI+2*RE_PWEL_NIMP-3*LAMBDA),                   \
	  /* transistor // */                                                \
          2*RE_NIMP_ACTI+2*RE_PWEL_NIMP-LAMBDA),                             \
	  /* contact polarisation */                                         \
	  RW_VIA0+2*RE_ACTI_VIA0+2*RE_PTIE_ACTI+2*RE_PWEL_PTIE-4*LAMBDA),    \
	  /* polarisation run _|_ */                                         \
          2*DL_ACTI+2*RE_PTIE_ACTI+2*RE_PWEL_PTIE-2*LAMBDA),                 \
	  /* polarisation run // */                                          \
          RW_ACTI+2*RE_PTIE_ACTI+2*RE_PWEL_PTIE-3*LAMBDA)                    \
        )

void
  main()
{
  float LAMBDA;

fprintf(stderr,"                  @@@@@@@                       @@@@@@     \n");
fprintf(stderr,"                    @@   @@                       @@       \n");
fprintf(stderr,"                    @@    @@                      @@       \n");
fprintf(stderr,"                    @@    @@ @@@ @@@     @@@      @@       \n");
fprintf(stderr,"                    @@   @@   @@@  @@  @@   @@    @@       \n");
fprintf(stderr,"                    @@@@@     @@   @@ @@     @@   @@       \n");
fprintf(stderr,"                    @@        @@      @@     @@   @@       \n");
fprintf(stderr,"                    @@        @@      @@     @@   @@       \n");
fprintf(stderr,"                    @@        @@      @@     @@   @@      @\n");
fprintf(stderr,"                    @@        @@       @@   @@    @@     @ \n");
fprintf(stderr,"                  @@@@@@     @@@@        @@@    @@@@@@@@@@ \n");
fprintf(stderr,"                                                           \n");
fprintf(stderr,"                               rds file maker\n");
fprintf(stderr,"                                                           \n");
fprintf(stderr,"                   Alliance CAD System 3.0,       prol 1.0 \n");
fprintf(stderr,"                   Copyright (c) 1994, MASI, CAO-VLSI Team \n");
fprintf(stderr,"                   E-mail support:    cao-vlsi@masi.ibp.fr \n");
fprintf(stderr,"                                                           \n");
fprintf(stderr,"           rules get in stdin stream, and rds file write out stdout\n");
fprintf(stderr,"                                                           \n");

  parse();

  printf("DEFINE PHYSICAL_GRID  % 7.3f\n\n", GRID);

  printf("DEFINE LAMBDA         % 7.3f\n\n", LAMBDA = lambda_def());

  printf("TABLE MBK_TO_RDS_SEGMENT\n\n");

    printf("  NWELL  RDS_NWELL VW % 7.3f % 7.3f    0.000  ALL\n",
		       DL_NWEL,
                       DW_NWEL-ODD_LAMBDA);

    printf("  PWELL  RDS_PWELL VW % 7.3f % 7.3f    0.000  ALL\n",
                       DL_PWEL,
                       DW_PWEL-ODD_LAMBDA);
		       
/***************/
/* DIFFUSION N */
/***************/

    printf("  NDIF   RDS_NDIF  VW % 7.3f % 7.3f    0.000  EXT\\\n",
		       DL_ACTI,
		       DW_ACTI-ODD_LAMBDA);
    printf("         RDS_ACTIV VW % 7.3f % 7.3f    0.000  DRC\\\n",
		       DL_ACTI,
		       DW_ACTI-ODD_LAMBDA);
    printf("         RDS_NIMP  VW % 7.3f % 7.3f    0.000  DRC\\\n",
		       DL_ACTI+RE_NIMP_ACTI,
		       DW_ACTI+RE_NIMP_ACTI*2-ODD_LAMBDA);
    printf("         RDS_PWELL VW % 7.3f % 7.3f    0.000  DRC  \n",
		       DL_ACTI+RE_NIMP_ACTI+RE_PWEL_NIMP,
		       DW_ACTI+(RE_NIMP_ACTI+RE_PWEL_NIMP)*2-ODD_LAMBDA);

/***************/
/* DIFFUSION P */
/***************/

    printf("  PDIF   RDS_PDIF  VW % 7.3f % 7.3f    0.000  EXT\\\n",
		       DL_ACTI,
		       DW_ACTI-ODD_LAMBDA);
    printf("         RDS_ACTIV VW % 7.3f % 7.3f    0.000  DRC\\\n",
		       DL_ACTI,
		       DW_ACTI-ODD_LAMBDA);
    printf("         RDS_PIMP  VW % 7.3f % 7.3f    0.000  DRC\\\n",
		       DL_ACTI+RE_PIMP_ACTI,
		       DW_ACTI+RE_PIMP_ACTI*2-ODD_LAMBDA);
    printf("         RDS_NWELL VW % 7.3f % 7.3f    0.000  DRC  \n",
		       DL_ACTI+RE_PIMP_ACTI+RE_NWEL_PIMP,
		       DW_ACTI+(RE_PIMP_ACTI+RE_NWEL_PIMP)*2-ODD_LAMBDA);

/**********************/
/* PRISE DE CAISSON N */
/**********************/

    printf("  NTIE   RDS_NTIE  VW % 7.3f % 7.3f    0.000  EXT\\\n",
		       DL_ACTI,
		       DW_ACTI-ODD_LAMBDA);
    printf("         RDS_ACTIV VW % 7.3f % 7.3f    0.000  DRC\\\n",
		       DL_ACTI,
		       DW_ACTI-ODD_LAMBDA);
    printf("         RDS_NIMP  VW % 7.3f % 7.3f    0.000  DRC\\\n",
		       DL_ACTI+RE_NTIE_ACTI,
		       DW_ACTI+RE_NTIE_ACTI*2-ODD_LAMBDA);
    printf("         RDS_NWELL VW % 7.3f % 7.3f    0.000  DRC  \n",
		       DL_ACTI+RE_NTIE_ACTI+RE_NWEL_NTIE,
		       DW_ACTI+(RE_NTIE_ACTI+RE_NWEL_NTIE)*2-ODD_LAMBDA);


/**********************/
/* PRISE DE CAISSON P */
/**********************/

    printf("  PTIE   RDS_PTIE  VW % 7.3f % 7.3f    0.000  EXT\\\n",
		       DL_ACTI,
		       DW_ACTI-ODD_LAMBDA);
    printf("         RDS_ACTIV VW % 7.3f % 7.3f    0.000  DRC\\\n",
		       DL_ACTI,
		       DW_ACTI-ODD_LAMBDA);
    printf("         RDS_PIMP  VW % 7.3f % 7.3f    0.000  DRC\\\n",
		       DL_ACTI+RE_PTIE_ACTI,
		       DW_ACTI+RE_PTIE_ACTI*2-ODD_LAMBDA);
    printf("         RDS_PWELL VW % 7.3f % 7.3f    0.000  DRC  \n",
		       DL_ACTI+RE_PTIE_ACTI+RE_PWEL_PTIE,
		       DW_ACTI+(RE_PTIE_ACTI+RE_PWEL_PTIE)*2-ODD_LAMBDA);


/****************/
/* TRANSISTOR N */
/****************/

    printf("  NTRANS RDS_GATE  VW % 7.3f % 7.3f    0.000  EXT\\\n",
		       0.0,
		       DW_POLY-ODD_LAMBDA);
    printf("         RDS_NDIF LCW % 7.3f % 7.3f  % 7.3f  EXT\\\n",
		       DL_ACTI_TRANS,
		       (SW_DIFT*LAMBDA-SW_POLX*LAMBDA)/2-ODD_LAMBDA,
                       grid_sup(DW_POLY/2));
    printf("         RDS_NDIF RCW % 7.3f % 7.3f  % 7.3f  EXT\\\n",
		       DL_ACTI_TRANS,
		       (SW_DIFT*LAMBDA-SW_POLX*LAMBDA)/2-ODD_LAMBDA,
                       grid_sup(DW_POLY/2));
    printf("         RDS_POLY  VW % 7.3f % 7.3f    0.000  DRC\\\n",
		       0.0,
		       DW_POLY-ODD_LAMBDA);
    printf("         RDS_ACTIV VW % 7.3f % 7.3f    0.000  DRC\\\n",
		       DL_ACTI_TRANS,
		       SW_DIFT*LAMBDA-SW_POLX*LAMBDA-ODD_LAMBDA);
    printf("         RDS_NIMP  VW % 7.3f % 7.3f    0.000  DRC\\\n",
		       DL_ACTI_TRANS+RE_NIMP_ACTI,
		       SW_DIFT*LAMBDA-SW_POLX*LAMBDA+RE_NIMP_ACTI*2-ODD_LAMBDA);
    printf("         RDS_PWELL VW % 7.3f % 7.3f    0.000  DRC\n",
		       DL_ACTI_TRANS+RE_NIMP_ACTI+RE_PWEL_NIMP,
		       SW_DIFT*LAMBDA-SW_POLX*LAMBDA+
		       (RE_NIMP_ACTI+RE_PWEL_NIMP)*2-ODD_LAMBDA);


/****************/
/* TRANSISTOR P */
/****************/

    printf("  PTRANS RDS_GATE  VW % 7.3f % 7.3f    0.000  EXT\\\n",
		       0.0,
		       DW_POLY-ODD_LAMBDA);
    printf("         RDS_PDIF LCW % 7.3f % 7.3f  % 7.3f  EXT\\\n",
		       DL_ACTI_TRANS,
		       (SW_DIFT*LAMBDA-SW_POLX*LAMBDA)/2-ODD_LAMBDA,
                       grid_sup(DW_POLY/2));
    printf("         RDS_PDIF RCW % 7.3f % 7.3f  % 7.3f  EXT\\\n",
		       DL_ACTI_TRANS,
		       (SW_DIFT*LAMBDA-SW_POLX*LAMBDA)/2-ODD_LAMBDA,
                       grid_sup(DW_POLY/2));
    printf("         RDS_POLY  VW % 7.3f % 7.3f    0.000  DRC\\\n",
		       0.0,
		       DW_POLY-ODD_LAMBDA);
    printf("         RDS_ACTIV VW % 7.3f % 7.3f    0.000  DRC\\\n",
		       DL_ACTI_TRANS,
		       SW_DIFT*LAMBDA-SW_POLX*LAMBDA-ODD_LAMBDA);
    printf("         RDS_PIMP  VW % 7.3f % 7.3f    0.000  DRC\\\n",
		       DL_ACTI_TRANS+RE_PIMP_ACTI,
		       SW_DIFT*LAMBDA-SW_POLX*LAMBDA+RE_PIMP_ACTI*2-ODD_LAMBDA);
    printf("         RDS_NWELL VW % 7.3f % 7.3f    0.000  DRC\n",
		       DL_ACTI_TRANS+RE_PIMP_ACTI+RE_NWEL_PIMP,
		       SW_DIFT*LAMBDA-SW_POLX*LAMBDA+
		       (RE_PIMP_ACTI+RE_NWEL_PIMP)*2-ODD_LAMBDA);


    printf("  POLY   RDS_POLY  VW % 7.3f % 7.3f    0.000  ALL\n",
                       DL_POLY,
                       DW_POLY-ODD_LAMBDA);
    printf("  ALU1   RDS_ALU1  VW % 7.3f % 7.3f    0.000  ALL\n",
                       DL_ALU1,
                       DW_ALU1-ODD_LAMBDA);
    printf("  ALU2   RDS_ALU2  VW % 7.3f % 7.3f    0.000  ALL\n",
                       DL_ALU2,
                       DW_ALU2-ODD_LAMBDA);
if (LEVELS==3) {
    printf("  ALU3   RDS_ALU3  VW % 7.3f % 7.3f    0.000  ALL\n",
                       DL_ALU3,
                       DW_ALU3-ODD_LAMBDA);
}

  printf("END\n");

  printf("TABLE MBK_TO_RDS_CONNECTOR\n");

    printf("  POLY   RDS_POLY  % 7.3f % 7.3f\n", 
                       DL_POLY, 
                       DW_POLY-ODD_LAMBDA);
    printf("  ALU1   RDS_ALU1  % 7.3f % 7.3f\n", 
                       DL_ALU1, 
                       DW_ALU1-ODD_LAMBDA);
    printf("  ALU2   RDS_ALU2  % 7.3f % 7.3f\n", 
                       DL_ALU2, 
                       DW_ALU2-ODD_LAMBDA);
if (LEVELS==3) {
    printf("  ALU3   RDS_ALU3  % 7.3f % 7.3f\n", 
                       DL_ALU3, 
                       DW_ALU3-ODD_LAMBDA);
}

  printf("END\n");

  printf("TABLE MBK_TO_RDS_REFERENCE\n");

    printf("  REF_REF  RDS_REF % 7.3f\n", RW_ALU1);
    printf("  REF_CON  RDS_REF % 7.3f\n", RW_ALU1);

  printf("END\n");

  printf("TABLE MBK_TO_RDS_VIA\n");

    printf("  CONT_BODY_N              \\\n");
    printf("    RDS_ALU1    % 7.3f  ALL\\\n", RW_VIA0+2*RE_ALU1_VIA0);
    printf("    RDS_CONT    % 7.3f  ALL\\\n", RW_VIA0);
    printf("    RDS_ACTIV   % 7.3f  DRC\\\n", RW_VIA0+2*RE_ACTI_VIA0);
    printf("    RDS_NIMP    % 7.3f  DRC\\\n", RW_VIA0+2*RE_ACTI_VIA0+
					      2*RE_NTIE_ACTI);
    printf("    RDS_NWELL   % 7.3f  DRC\\\n", RW_VIA0+2*RE_ACTI_VIA0+
					      2*RE_NTIE_ACTI+2*RE_NWEL_NTIE);
    printf("    RDS_NTIE    % 7.3f  EXT  \n", RW_VIA0+2*RE_ACTI_VIA0);

    printf("  CONT_BODY_P              \\\n");
    printf("    RDS_ALU1    % 7.3f  ALL\\\n", RW_VIA0+2*RE_ALU1_VIA0);
    printf("    RDS_CONT    % 7.3f  ALL\\\n", RW_VIA0);
    printf("    RDS_ACTIV   % 7.3f  DRC\\\n", RW_VIA0+2*RE_ACTI_VIA0);
    printf("    RDS_PIMP    % 7.3f  DRC\\\n", RW_VIA0+2*RE_ACTI_VIA0+
					      2*RE_PTIE_ACTI);
    printf("    RDS_PWELL   % 7.3f  DRC\\\n", RW_VIA0+2*RE_ACTI_VIA0+
					      2*RE_NTIE_ACTI+2*RE_NWEL_NTIE);
    printf("    RDS_PTIE    % 7.3f  EXT  \n", RW_VIA0+2*RE_ACTI_VIA0);

    printf("  CONT_DIF_N               \\\n");
    printf("    RDS_ALU1    % 7.3f  ALL\\\n", RW_VIA0+2*RE_ALU1_VIA0);
    printf("    RDS_CONT    % 7.3f  ALL\\\n", RW_VIA0);
    printf("    RDS_ACTIV   % 7.3f  DRC\\\n", RW_VIA0+2*RE_ACTI_VIA0);
    printf("    RDS_NIMP    % 7.3f  DRC\\\n", RW_VIA0+2*RE_ACTI_VIA0+
					      2*RE_NIMP_ACTI);
    printf("    RDS_PWELL   % 7.3f  DRC\\\n", RW_VIA0+2*RE_ACTI_VIA0+
					      2*RE_NIMP_ACTI+2*RE_PWEL_NIMP);
    printf("    RDS_NDIF    % 7.3f  EXT  \n", RW_VIA0+2*RE_ACTI_VIA0);

    printf("  CONT_DIF_P               \\\n");
    printf("    RDS_ALU1    % 7.3f  ALL\\\n", RW_VIA0+2*RE_ALU1_VIA0);
    printf("    RDS_CONT    % 7.3f  ALL\\\n", RW_VIA0);
    printf("    RDS_ACTIV   % 7.3f  DRC\\\n", RW_VIA0+2*RE_ACTI_VIA0);
    printf("    RDS_PIMP    % 7.3f  DRC\\\n", RW_VIA0+2*RE_ACTI_VIA0+
					      2*RE_PIMP_ACTI);
    printf("    RDS_NWELL   % 7.3f  DRC\\\n", RW_VIA0+2*RE_ACTI_VIA0+
				  	      2*RE_PIMP_ACTI+2*RE_NWEL_PIMP);
    printf("    RDS_PDIF    % 7.3f  EXT  \n", RW_VIA0+2*RE_ACTI_VIA0);

    printf("  CONT_POLY                \\\n");
    printf("    RDS_ALU1    % 7.3f  ALL\\\n", RW_VIA0+2*RE_ALU1_VIA0);
    printf("    RDS_CONT    % 7.3f  ALL\\\n", RW_VIA0);
    printf("    RDS_POLY    % 7.3f  ALL  \n", RW_VIA0+2*RE_POLY_VIA0);

    printf("  CONT_VIA                 \\\n");
    printf("    RDS_ALU1    % 7.3f  ALL\\\n", RW_VIA1+2*RE_ALU1_VIA1);
    printf("    RDS_VIA1    % 7.3f  ALL\\\n", RW_VIA1);
    printf("    RDS_ALU2    % 7.3f  ALL  \n", RW_VIA1+2*RE_ALU2_VIA1);

if (LEVELS==3) {
    printf("  CONT_VIA2                \\\n");
    printf("    RDS_ALU2    % 7.3f  ALL\\\n", RW_VIA2+2*RE_ALU2_VIA2);
    printf("    RDS_VIA2    % 7.3f  ALL\\\n", RW_VIA2);
    printf("    RDS_ALU3    % 7.3f  ALL  \n", RW_VIA2+2*RE_ALU3_VIA2);
}

    printf("  C_X_N                    \\\n");
    printf("    RDS_POLY    % 7.3f  DRC\\\n", RW_POLY);
    printf("    RDS_ACTIV   % 7.3f  DRC\\\n", SW_DIFT*LAMBDA);
    printf("    RDS_NIMP    % 7.3f  DRC\\\n", SW_DIFT*LAMBDA+2*RE_NIMP_ACTI);
    printf("    RDS_PWELL   % 7.3f  DRC\\\n", SW_DIFT*LAMBDA+2*RE_NIMP_ACTI+
					      2*RE_PWEL_NIMP);
    printf("    RDS_GATE    % 7.3f  EXT\\\n", RW_POLY);
    printf("    RDS_NDIF    % 7.3f  EXT  \n", SW_DIFT*LAMBDA);

    printf("  C_X_P                    \\\n");
    printf("    RDS_POLY    % 7.3f  DRC\\\n", RW_POLY);
    printf("    RDS_ACTIV   % 7.3f  DRC\\\n", SW_DIFT*LAMBDA);
    printf("    RDS_PIMP    % 7.3f  DRC\\\n", SW_DIFT*LAMBDA+2*RE_PIMP_ACTI);
    printf("    RDS_NWELL   % 7.3f  DRC\\\n", SW_DIFT*LAMBDA+2*RE_PIMP_ACTI+
					   2*RE_NWEL_PIMP);
    printf("    RDS_GATE    % 7.3f  EXT\\\n", RW_POLY);
    printf("    RDS_PDIF    % 7.3f  EXT  \n", SW_DIFT*LAMBDA);

  printf("END\n");

  printf("TABLE CIF_LAYER \n");

    printf("  RDS_NWELL    %s\n", CIF_NWEL);
    printf("  RDS_POLY     %s\n", CIF_POLY);
    printf("  RDS_CONT     %s\n", CIF_VIA0);
    printf("  RDS_ALU1     %s\n", CIF_ALU1);
    printf("  RDS_TALU1    %s\n", CIF_TALU1);
    printf("  RDS_VIA1     %s\n", CIF_VIA1);
    printf("  RDS_ALU2     %s\n", CIF_ALU2);
    printf("  RDS_TALU2    %s\n", CIF_TALU2);
    printf("  RDS_ACTIV    %s\n", CIF_ACTI);
    printf("  RDS_NIMP     %s\n", CIF_NIMP);
    printf("  RDS_PIMP     %s\n", CIF_PIMP);
    printf("  RDS_CPAS     %s\n", CIF_CPAS);
if (LEVELS==3) {
    printf("  RDS_ALU3     %s\n", CIF_ALU3);
    printf("  RDS_VIA2     %s\n", CIF_VIA2);
}

  printf("END\n");

  printf("TABLE GDS_LAYER\n");

    printf("  RDS_NWELL    %d\n", GDS_NWEL);
    printf("  RDS_POLY     %d\n", GDS_POLY);
    printf("  RDS_CONT     %d\n", GDS_VIA0);
    printf("  RDS_ALU1     %d\n", GDS_ALU1);
    printf("  RDS_VIA1     %d\n", GDS_VIA1);
    printf("  RDS_ALU2     %d\n", GDS_ALU2);
    printf("  RDS_ACTIV    %d\n", GDS_ACTI);
    printf("  RDS_NIMP     %d\n", GDS_NIMP);
    printf("  RDS_PIMP     %d\n", GDS_PIMP);
    printf("  RDS_CPAS     %d\n", GDS_CPAS);
if (LEVELS==3) {                
    printf("  RDS_ALU3     %d\n", GDS_ALU3);
    printf("  RDS_VIA2     %d\n", GDS_VIA2);
}                               
                                
  printf("END\n");

  printf("TABLE S2R_OVERSIZE_DENOTCH\n");

    printf("  RDS_NWELL   % 7.3f\n", RD_NWEL/2);
    printf("  RDS_PWELL   % 7.3f\n", RD_PWEL/2);
    printf("  RDS_POLY    % 7.3f\n", RD_POLY/2-GRID);
    printf("  RDS_ALU1    % 7.3f\n", RD_ALU1/2-GRID);
    printf("  RDS_ALU2    % 7.3f\n", RD_ALU2/2-GRID);
    printf("  RDS_ACTIV   % 7.3f\n", RD_ACTI/2-GRID);
    printf("  RDS_NIMP    % 7.3f\n", RD_NIMP/2);
    printf("  RDS_PIMP    % 7.3f\n", RD_PIMP/2);
if (LEVELS==3) {
    printf("  RDS_ALU3    % 7.3f\n", RD_ALU3/2-GRID);
}

  printf("END\n");

  printf("TABLE S2R_BLOC_RING_WIDTH\n");

    printf("  RDS_NWELL   % 7.3f\n", RD_NWEL);
    printf("  RDS_PWELL   % 7.3f\n", RD_PWEL);
    printf("  RDS_POLY    % 7.3f\n", RD_POLY);
    printf("  RDS_ALU1    % 7.3f\n", RD_ALU1);
    printf("  RDS_ALU2    % 7.3f\n", RD_ALU2);
    printf("  RDS_ACTIV   % 7.3f\n", RD_ACTI);
    printf("  RDS_NIMP    % 7.3f\n", RD_NIMP);
    printf("  RDS_PIMP    % 7.3f\n", RD_PIMP);
if (LEVELS==3) {
    printf("  RDS_ALU3    % 7.3f\n", RD_ALU3);
}

  printf("END\n");

  printf("TABLE S2R_MINIMUM_LAYER_WIDTH \n");

    printf("  RDS_NWELL   % 7.3f\n", RW_NWEL);
    printf("  RDS_PWELL   % 7.3f\n", RW_PWEL);
    printf("  RDS_POLY    % 7.3f\n", RW_POLY);
    printf("  RDS_ALU1    % 7.3f\n", RW_ALU1);
    printf("  RDS_ALU2    % 7.3f\n", RW_ALU2);
    printf("  RDS_ACTIV   % 7.3f\n", RW_ACTI);
    printf("  RDS_NIMP    % 7.3f\n", RW_NIMP);
    printf("  RDS_PIMP    % 7.3f\n", RW_PIMP);
if (LEVELS==3) {
    printf("  RDS_ALU3    % 7.3f\n", RW_ALU3);
}

  printf("END\n");

  printf("TABLE S2R_POST_TREAT\n");

    printf("  RDS_NWELL        TREAT NULL     \n");
    printf("  RDS_PWELL        TREAT NULL     \n");
    printf("  RDS_POLY         TREAT NULL     \n");
    printf("  RDS_CONT       NOTREAT NULL     \n");
    printf("  RDS_ALU1         TREAT NULL     \n");
    printf("  RDS_VIA1       NOTREAT NULL     \n");
    printf("  RDS_ALU2         TREAT NULL     \n");
    printf("  RDS_ACTIV        TREAT NULL     \n");
    printf("  RDS_NIMP         TREAT RDS_PIMP \n");
    printf("  RDS_PIMP         TREAT RDS_NIMP \n");
    printf("  RDS_ABOX       NOTREAT NULL     \n");
if (LEVELS==3) {
    printf("  RDS_VIA2       NOTREAT NULL     \n");
    printf("  RDS_ALU3         TREAT NULL     \n");
}

  printf("END\n");

  printf("TABLE LYNX_TRANSISTOR\n");

    printf("NTRANS  NTRANS  C_X_N RDS_GATE RDS_NDIF\n");
    printf("PTRANS  PTRANS  C_X_P RDS_GATE RDS_PDIF\n");

  printf("END\n");

  printf("TABLE LYNX_GRAPH\n");

  printf("  RDS_NDIF   RDS_CONT   RDS_NDIF\n");
  printf("  RDS_PDIF   RDS_CONT   RDS_PDIF\n");
  printf("  RDS_NTIE   RDS_CONT   RDS_NTIE\n");
  printf("  RDS_PTIE   RDS_CONT   RDS_PTIE\n");
  printf("  RDS_POLY   RDS_CONT   RDS_GATE   RDS_POLY\n");
  printf("  RDS_GATE   RDS_POLY   RDS_GATE\n");
  printf("  RDS_CONT   RDS_PDIF   RDS_NDIF   RDS_POLY  RDS_PTIE   RDS_NTIE   RDS_ALU1  RDS_CONT\n");
  printf("  RDS_ALU1   RDS_CONT   RDS_VIA1   RDS_ALU1  RDS_REF\n");
  printf("  RDS_REF    RDS_CONT   RDS_VIA1   RDS_ALU1  RDS_REF\n");
  printf("  RDS_VIA1   RDS_ALU1   RDS_ALU2   RDS_VIA1\n");
  printf("  RDS_ALU2   RDS_VIA1   RDS_VIA2   RDS_ALU2\n");
  printf("  RDS_VIA2   RDS_ALU2   RDS_ALU3   RDS_VIA2\n");
  printf("  RDS_ALU3   RDS_VIA2   RDS_ALU3\n");

  printf("END\n");

  printf("TABLE LYNX_CAPA\n");

  printf("  RDS_POLY    % 7.3g % 7.3g\n",CAPA_S_POLY, CAPA_P_POLY);
  printf("  RDS_ALU1    % 7.3g % 7.3g\n",CAPA_S_ALU1, CAPA_P_ALU1);
  printf("  RDS_ALU2    % 7.3g % 7.3g\n",CAPA_S_ALU2, CAPA_P_ALU2);

  printf("END\n");

  exit(0);
}
